package cn.com.ty.lift.business.emergency.controller;


import cn.com.ty.lift.business.common.CountPage;
import cn.com.ty.lift.business.common.Judge;
import cn.com.ty.lift.business.common.Values;
import cn.com.ty.lift.business.emergency.dto.LiftFaultCategory;
import cn.com.ty.lift.business.emergency.dto.LiftFaultType;
import cn.com.ty.lift.business.emergency.dto.RepairRequest;
import cn.com.ty.lift.business.emergency.dto.RepairResponse;
import cn.com.ty.lift.business.emergency.entity.EmergencyRepair;
import cn.com.ty.lift.business.emergency.entity.ErRecordCost;
import cn.com.ty.lift.business.emergency.entity.ErRecordImg;
import cn.com.ty.lift.business.emergency.entity.LiftFault;
import cn.com.ty.lift.business.emergency.service.EmergencyRepairService;
import cn.com.ty.lift.business.emergency.service.ErRecordCostService;
import cn.com.ty.lift.business.emergency.service.ErRecordImgService;
import cn.com.ty.lift.business.emergency.service.LiftFaultService;
import cn.com.ty.lift.business.evaluation.dao.entity.Evaluation;
import cn.com.ty.lift.business.evaluation.service.EvaluationService;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.collection.IterUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 临修记录 前端控制器
 * </p>
 *
 * @author wcz
 * @since 2019-12-04
 */
@Slf4j
@RestController
@AllArgsConstructor
@RequestMapping("/emergency/emergency-repair")
public class EmergencyRepairController {

    private EmergencyRepairService emergencyRepairService;
    private ErRecordCostService erRecordCostService;
    private LiftFaultService liftFaultService;
    private EvaluationService evaluationService;
    private ErRecordImgService erRecordImgService;

    /**
     * @description 根据id查找
     * @date 2019/12/6 10:14
     * @param request
     * @return RestResponse
     */
    @PostMapping("findOne")
    public RestResponse findOne(@RequestBody RepairRequest request){
        Long id = request.getId();
        Judge.id(id);

        RepairResponse result = emergencyRepairService.selectByIdWithInfo(request);
        Judge.notNull(result);

        //查询相关的急修费用
        result.setErRecordCosts(erRecordCostService.listByErRecordId(id));
        //急修图片
        result.setErRecordImgs(erRecordImgService.listByErRecordId(id));
        //评价
        result.setEvaluation(evaluationService.findByRecord(id));
        //故障信息
        String faultPart = result.getFaultPart();
        String faultReason = result.getFaultReason();
        String faultNature = result.getFaultNature();
        String faultHandle = result.getFaultHandle();
        String faultDuty = result.getFaultDuty();
        if(StrUtil.isNotEmpty(faultPart)){
            List<Long> partIds = Arrays.asList(faultPart.split(",")).stream().map(s -> Long.parseLong(s.trim())).collect(Collectors.toList());
            result.setFaultParts(liftFaultService.listByIds(partIds));
        }
        if(StrUtil.isNotEmpty(faultReason)){
            List<Long> reasonIds = Arrays.asList(faultPart.split(",")).stream().map(s -> Long.parseLong(s.trim())).collect(Collectors.toList());
            result.setFaultReasons(liftFaultService.listByIds(reasonIds));
        }
        if(StrUtil.isNotEmpty(faultNature)){
            List<Long> natureIds = Arrays.asList(faultPart.split(",")).stream().map(s -> Long.parseLong(s.trim())).collect(Collectors.toList());
            result.setFaultNatures(liftFaultService.listByIds(natureIds));
        }
        if(StrUtil.isNotEmpty(faultHandle)){
            List<Long> handleIds = Arrays.asList(faultPart.split(",")).stream().map(s -> Long.parseLong(s.trim())).collect(Collectors.toList());
            result.setFaultHandles(liftFaultService.listByIds(handleIds));
        }
        if(StrUtil.isNotEmpty(faultDuty)){
            List<Long> dutyIds = Arrays.asList(faultPart.split(",")).stream().map(s -> Long.parseLong(s.trim())).collect(Collectors.toList());
            result.setFaultDuties(liftFaultService.listByIds(dutyIds));
        }
        return RestResponse.success(result);
    }

    /**
     * @description 新增急修
     * @param entity
     * @return RestResponse
     * @date 2019/12/9 9:43
     */
    @PostMapping("add")
    public RestResponse add(@Valid @RequestBody EmergencyRepair entity){
        Long mtCompanyId = entity.getMtCompanyId();
        Long liftId = entity.getLiftId();
        int repairing = emergencyRepairService.countRepairing(mtCompanyId,liftId);
        Judge.notTrue(repairing > 0,Values.Er.haveUnfinisedRepair);

        //派单时间 急修单创建时间
        entity.setHasEvaluate(0);
        entity.setAssignTime(LocalDateTime.now());
        entity.setStatus(Values.Er.STATUS_TO_REPAIRED);
        boolean result = emergencyRepairService.save(entity);
        return RestResponse.success(result);
    }

    /**
     * @description 分页查询急修中的 status = 2
     * @date 2019/12/9 9:39
     * @param 
     * @return 
     */
    @PostMapping("list")
    public RestResponse list(@RequestBody RepairRequest request){
        IPage<RepairResponse> pages = emergencyRepairService.pageByRepairing(request);
        Judge.notNull(pages);
        long count = pages.getTotal();
        CountPage<RepairResponse> countPage = new CountPage<>(count,pages);
        return RestResponse.success(countPage);
    }

    /**
     * @description 根据条件分页查询
     * @date 2019/12/9 9:39
     * @param 
     * @return 
     */
    @PostMapping("page")
    public RestResponse page(@RequestBody RepairRequest request){
        IPage<RepairResponse> pages = emergencyRepairService.pageByCondition(request);
        Judge.notNull(pages);
        long count = emergencyRepairService.countByCondition(request);
        CountPage<RepairResponse> countPage = new CountPage<>(count,pages);
        return RestResponse.success(countPage);
    }

    /**
     *  根据条件分页查询
     * @param
     * @return RestResponse
     */
    @PostMapping("pageWorker")
    public RestResponse pageWorker(@RequestBody RepairRequest request){
        Long workerId = request.getWorkerId();
        Judge.id(workerId);
        IPage<RepairResponse> pages = emergencyRepairService.pageByCondition(request);
        return RestResponse.success(pages);
    }

    /**
     * 急修转派
     * @param request
     * @return RestResponse
     */
    @PostMapping("transfer")
    public RestResponse transfer(@RequestBody RepairRequest request){
        Long id = request.getId();
        Long workerId = request.getWorkerId();
        Judge.ids(id,workerId);

        EmergencyRepair entity = emergencyRepairService.getById(id);
        Judge.notNull(entity);

        //如果已经接单，在急修中了不能被转派
        Judge.notTrue(Values.Er.statusRepairing(entity.getStatus()),Values.Er.repairHadTaked);
        //本急修单的负责维保工
        entity.setWorkerId1(workerId);

        boolean result = emergencyRepairService.saveOrUpdate(entity);
        return RestResponse.success(result);
    }

    /**
     * 关闭急修
     * @param request
     * @return RestResponse
     */
    @PostMapping("close")
    public RestResponse close(@RequestBody RepairRequest request){
        Long id = request.getId();
        Judge.id(id);

        String reason = request.getCloseReason();
        Judge.notNull(reason,Values.Er.missingReason);

        EmergencyRepair entity = emergencyRepairService.getById(id);
        Judge.notNull(entity);

        entity.setCloseReason(reason);
        //新状态（-1 暂停中，0 待修理；1 修理中；2 已完成 3 已关闭）
        entity.setStatus(Values.Er.STATUS_CLOSE);

        boolean result = emergencyRepairService.saveOrUpdate(entity);
        return RestResponse.success(result);
    }

    /**
     * app-其他角色获取自己关联的项目的急修任务
     * @param request
     * @return
     */
    @PostMapping("pageByUser")
    public RestResponse pageByUser(@RequestBody RepairRequest request){
        Long userId = request.getUserId();
        Judge.id(userId);
        IPage<RepairResponse> pages = emergencyRepairService.pageByUser(request);
        return RestResponse.success(pages);
    }

    /**
     * app-点击接单，保存接单时间到急修表的接单时间接口
     * 接单的时候就是签到
     * @param request
     * @return RestResponse
     */
    @PostMapping("taking")
    public RestResponse taking(@RequestBody RepairRequest request){
        Long id = request.getId();
        Judge.id(id);

        LocalDateTime takingTime = request.getTakingTime();
        Judge.notNull(takingTime,Values.Er.missingHandleDate);

        EmergencyRepair entity = emergencyRepairService.getById(id);
        Judge.notNull(entity);

        Judge.isTrue(Values.Er.statusToRepaired(entity.getStatus()),Values.Er.mustToRepair);
        entity.setTakingTime(takingTime);
        //todo:修改电梯的业务状态为急修中,如果存在维保中，要修改
        entity.setStatus(Values.Er.STATUS_REPAIRING);

        boolean result = emergencyRepairService.taking(entity);
        return RestResponse.success(result);
    }

    /**
     * app-点击到达，保存到达时间到急修表的到达时间中
     * @param request 请求参数
     * @return RestResponse
     */
    @PostMapping("arrive")
    public RestResponse arrive(@RequestBody RepairRequest request){
        Long id = request.getId();
        Judge.id(id);

        LocalDateTime arriveTime = request.getArriveTime();
        Judge.notNull(arriveTime,Values.Er.missingHandleDate);

        String position = request.getPosition();
        Judge.notNull(position,Values.Er.missionPosition);

        EmergencyRepair entity = emergencyRepairService.getById(id);
        Judge.notNull(entity);

        Judge.isTrue(Values.Er.statusRepairing(entity.getStatus()), Values.Er.mustInRepairing);
        entity.setArriveTime(arriveTime);

        boolean result = emergencyRepairService.updateById(entity);
        return RestResponse.success(result);
    }

    /**
     * app-停梯时间+安全确认，保存停梯时间+安全确认项到急修表中
     * @param request 请求参数
     * @return RestResponse
     */
    @PostMapping("stop")
    public RestResponse stopLift(@RequestBody RepairRequest request){
        Long id = request.getId();
        Judge.id(id);

        LocalDateTime stopDate = request.getStopDate();
        Judge.notNull(stopDate,Values.Er.missingHandleDate);
        String safetyConfirm = request.getSafetyConfirm();
        //检查安全确认项值
        Judge.notNull(safetyConfirm,Values.Er.safetyConfirm);

        EmergencyRepair entity = emergencyRepairService.getById(id);
        Judge.notNull(entity);

        Judge.isTrue(Values.Er.statusRepairing(entity.getStatus()),Values.Er.mustInRepairing);
        entity.setStopDate(stopDate);
        entity.setSafetyConfirm(safetyConfirm);

        boolean result = emergencyRepairService.updateById(entity);
        return RestResponse.success(result);
    }

    /**
     * 新增急修收费单项
     * @param request 请求参数
     * @return RestResponse
     */
    @PostMapping("cost/add")
    public RestResponse costAdd(@RequestBody RepairRequest request){
        Long id = request.getId();
        Judge.id(id);

        EmergencyRepair repair = emergencyRepairService.getById(id);
        Judge.notNull(repair);

        List<ErRecordCost> erRecordCosts = request.getErRecordCosts();
        Judge.notNull(erRecordCosts,Values.Er.missingCost);
        BigDecimal costTotal = new BigDecimal(0);
        erRecordCosts.forEach(item -> {
            costTotal.add(item.getCostMoney());
        });
        repair.setCostTotal(costTotal);

        boolean result = emergencyRepairService.costAdd(repair, erRecordCosts);
        return RestResponse.success(result);
    }

    /**
     * 删除急修下的收费项
     * @param request 请求参数
     * @return RestResponse
     */
    @PostMapping("cost/delete")
    public RestResponse costDelete(@RequestBody RepairRequest request){
        Long id = request.getId();
        Long erCostItemId = request.getErCostItemId();
        Judge.ids(id,erCostItemId);

        EmergencyRepair repair = emergencyRepairService.getById(id);
        Judge.notNull(repair);

        ErRecordCost erRecordCost = erRecordCostService.findByRecordAndId(id,erCostItemId);
        Judge.notNull(erRecordCost);

        BigDecimal costTotal = repair.getCostTotal();
        BigDecimal costMoney = erRecordCost.getCostMoney();
        Judge.notNull(costTotal,Values.Er.incorrectAmount);
        Judge.notNull(costMoney,Values.Er.incorrectAmount);
        //收费项的金额不能大于急修单的总金额
        Judge.notTrue(costTotal.compareTo(costMoney) == -1,Values.Er.incorrectAmount);

        repair.setCostTotal(repair.getCostTotal().subtract(erRecordCost.getCostMoney()));

        boolean result = emergencyRepairService.costDelete(repair, erRecordCost);
        return RestResponse.success(result);
    }

    /**
     * 修改急修单下的急修收费项
     * @param request
     * @return RestResponse
     */
    @PostMapping("cost/modify")
    public RestResponse costModify(@RequestBody RepairRequest request){
        Long id = request.getId();
        Judge.id(id);

        EmergencyRepair repair = emergencyRepairService.getById(id);
        Judge.notNull(repair);

        List<ErRecordCost> erRecordCosts = request.getErRecordCosts();
        Judge.notNull(erRecordCosts);
        BigDecimal costTotal = new BigDecimal(0);
        erRecordCosts.forEach(item -> {
            costTotal.add(item.getCostMoney());
        });
        repair.setCostTotal(costTotal);

        boolean result = emergencyRepairService.costModify(repair, erRecordCosts);
        return RestResponse.success(result);
    }

    /**
     * 根据电梯类型查询故障信息（集合结构）
     * @param request
     * @return RestResponse
     */
    @PostMapping("fault/list")
    public RestResponse faultList(@RequestBody RepairRequest request){
        Integer liftCategory = request.getLiftCategory();
        if(null == liftCategory || liftCategory <= 0){
            return RestResponse.success(liftFaultService.list());
        }
        List<LiftFault> lists = liftFaultService.listByLiftCategory(liftCategory);
        return RestResponse.success(lists);
    }

    /**
     * 根据电梯类型获取急修故障信息列表(树结构)
     * @param request
     * @return RestResponse
     */
    @PostMapping("fault/tree")
    public RestResponse faultTree(@RequestBody RepairRequest request) {
        Integer liftCategory = request.getLiftCategory();
        if(null == liftCategory || liftCategory <= 0){
            List<LiftFault> liftFaults = liftFaultService.list();
            return assemble(liftFaults);
        }else{
            List<LiftFault> liftFaults = liftFaultService.listByLiftCategory(liftCategory);
            return assemble(liftFaults);
        }
    }

    /**
     * 根据故障列表组装tree
     * @param liftFaults 电梯故障list
     * @return RestResponse
     */
    private RestResponse assemble(List<LiftFault> liftFaults){
        Judge.notNull(liftFaults,Values.Er.missingFault);
        List<String> categories = liftFaults.stream().map(LiftFault::getFaultCategory).distinct().collect(Collectors.toList());
        Judge.notNull(categories,Values.Er.missingFault);
        List<LiftFaultCategory> liftFaultCategories = new ArrayList<>();
        categories.forEach(category -> {
            List<String> types = liftFaults.stream().filter(liftFault -> (
                    StrUtil.equals(category,liftFault.getFaultCategory())
            )).map(LiftFault::getFaultType).distinct().collect(Collectors.toList());

            List<LiftFaultType> liftFaultTypes = new ArrayList<>();
            if (ObjectUtil.isNotEmpty(types)) {
                types.forEach(type -> {
                    List<LiftFault> faults = liftFaults.stream().filter(liftFault -> (
                            StrUtil.equals(category, liftFault.getFaultCategory()) &&
                                    StrUtil.equals(type, liftFault.getFaultType())
                    )).distinct().collect(Collectors.toList());
                    liftFaultTypes.add(new LiftFaultType(type,faults));
                });
            }
            liftFaultCategories.add(new LiftFaultCategory(category,liftFaultTypes));
        });
        return RestResponse.success(liftFaultCategories);
    }

    /**
     * 急修评价
     * @param entity 封装评价的参数model
     * @return RestResponse
     */
    @PostMapping("evaluate")
    public RestResponse evaluate(@Valid @RequestBody Evaluation entity){
        Long recordId = entity.getRecordId();
        EmergencyRepair repair = emergencyRepairService.getById(recordId);
        Judge.notNull(repair);

        Judge.isTrue(Values.Er.statusFinish(repair.getStatus()),Values.Er.mustFinish);
        Judge.notTrue(repair.getHasEvaluate() == 1,Values.Er.hadEvaluate);

        //来源（1 维保；2 急修）
        entity.setSource(2);
        entity.setLiftId(repair.getLiftId());
        entity.setPostDate(LocalDateTime.now());

        boolean result = emergencyRepairService.evaluate(repair,entity);
        return RestResponse.success(result);
    }

    /**
     * 生成急修单，添加主要，次要急修人签名
     * @param request 请求参数model
     * @return RestResponse
     */
    @PostMapping("repairOrder")
    public RestResponse repairOrder(@RequestBody RepairRequest request){
        Long id = request.getId();
        Judge.id(id);

        LocalDateTime recoveryDate = request.getRecoveryDate();
        Judge.notNull(recoveryDate,Values.Er.missingHandleDate);

        //检查主签名
        String mainSign = request.getMainSign();
        String secondSign = request.getSecondSign();
        Judge.notNull(mainSign,Values.Er.missingSign);

        List<String> erRecordImg = request.getErRecordImg();
        Judge.isTrue(IterUtil.isNotEmpty(erRecordImg),Values.Er.missingImage);

        EmergencyRepair entity = emergencyRepairService.getById(id);
        Judge.notNull(entity);
        Judge.isTrue(Values.Er.statusRepairing(entity.getStatus()),Values.Er.mustInRepairing);
        List<ErRecordImg> erRecordImgs = new ArrayList<>();
        for (int i = 0; i < erRecordImg.size(); i++) {
            ErRecordImg item = new ErRecordImg();
            item.setErRecordId(id);
            item.setImgUrl(erRecordImg.get(i));
            item.setSort(i);
            erRecordImgs.add(item);
        }
        entity.setMainSign(mainSign);
        entity.setSecondSign(secondSign);
        entity.setFieldDescription(request.getFieldDescription());
        entity.setFaultPart(request.getFaultPart());
        entity.setFaultReason(request.getFaultReason());
        entity.setFaultHandle(request.getFaultHandle());
        entity.setFaultNature(request.getFaultNature());
        entity.setFaultDuty(request.getFaultDuty());
        entity.setRecoveryDate(recoveryDate);
        //修改状态已完成
        entity.setStatus(Values.Er.STATUS_FINISH);

        boolean result = emergencyRepairService.repairOrder(entity,erRecordImgs);
        return RestResponse.success(result);
    }

    @PostMapping("listImages")
    public RestResponse listImages(@RequestBody RepairRequest request){
        Long id = request.getId();
        Judge.id(id);
        List<ErRecordImg> lists = erRecordImgService.listByErRecordId(id);
        Judge.notNull(lists);
        return RestResponse.success(lists);
    }

    /**
     * 电梯管理中查询急修记录
     * @param request
     * @return
     */
    @PostMapping("listRepair")
    public RestResponse listRepair(@RequestBody RepairRequest request){
        Long mtCompanyId = request.getMtCompanyId();
        Long liftId = request.getLiftId();
        Long projectId = request.getProjectId();
        Judge.ids(liftId,mtCompanyId,projectId);
        IPage<RepairResponse> pages = emergencyRepairService.pageByCompanyLiftProject(request);
        return RestResponse.success(pages);
    }
}
