package cn.com.ty.lift.business.maintenance.controller;

import cn.com.ty.lift.business.common.CountPage;
import cn.com.ty.lift.business.common.Judge;
import cn.com.ty.lift.business.maintenance.dao.entity.MaintenanceOption;
import cn.com.ty.lift.business.maintenance.dao.entity.MaintenanceRecord;
import cn.com.ty.lift.business.maintenance.dao.entity.model.request.MtPlanRequest;
import cn.com.ty.lift.business.maintenance.dao.entity.model.request.MtRecordRequest;
import cn.com.ty.lift.business.maintenance.dao.entity.model.response.MtOptionTree;
import cn.com.ty.lift.business.maintenance.dao.entity.model.response.MtPlanResponse;
import cn.com.ty.lift.business.maintenance.dao.entity.model.response.MtRecordResponse;
import cn.com.ty.lift.business.maintenance.service.*;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.google.common.base.Splitter;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;
import java.util.stream.Collectors;

/**
　* <p>
  * 维保管理-日常保养
  * <p>
  * @author wcz
　* @since 2020/1/17 17:24
　*/
@Slf4j
@AllArgsConstructor
@RestController
@RequestMapping("maintenance/record")
public class MaintenanceRecordController {

    private MaintenancePlanService maintenancePlanService;
    private MaintenanceRecordService maintenanceRecordService;
    private MtRecordCostService mtRecordCostService;
    private MaintenanceOptionService maintenanceOptionService;
    private MtRecordImgService mtRecordImgService;

    /**
     * 根据条件分页查询维保-日常保养
     * @param request
     * @return
     */
    @PostMapping("pageRecord")
    public RestResponse pageRecord(@RequestBody MtRecordRequest request){
        Long mtCompanyId = request.getMtCompanyId();
        Judge.id(mtCompanyId);
        IPage<MtRecordResponse> pages = maintenanceRecordService.pageRecordByCondition(request);
        Judge.notNull(pages);
        long count = maintenanceRecordService.countRecordByCondition(request);
        CountPage<MtRecordResponse> countPage = new CountPage<>(count,pages);
        return RestResponse.success(countPage);
    }

    /**
     * 根据条件分页查询超期的维保计划
     * @param request
     * @return
     */
    @PostMapping("pagePlan")
    public RestResponse pagePlan(@RequestBody MtPlanRequest request){
        Long mtCompanyId = request.getMtCompanyId();
        Judge.id(mtCompanyId);
        IPage<MtPlanResponse> pages = maintenancePlanService.pagePlanByCondition(request);
        Judge.notNull(pages);
        long count = maintenancePlanService.countPlanByCondition(request);
        CountPage<MtPlanResponse> countPage = new CountPage<>(count,pages);
        return RestResponse.success(countPage);
    }

    /**
     * 根据id查找
     * @param request
     * @return
     */
    @PostMapping("one")
    public RestResponse one(@RequestBody MtRecordRequest request){
        Long id = request.getId();
        Judge.id(id);
        MtRecordResponse entity = maintenanceRecordService.findByIdWithInfo(request);
        Judge.notNull(entity);
        /**
         * 维保项id:0/1/2, 0:无需保养，1：已保养，2：需要更换
         * 保养项目  用键值对的方式存储，如(1:0,2:1,3:1...)
         */
        String maintenanceOption = entity.getMaintenanceOption();
        if(StrUtil.isNotEmpty(maintenanceOption)){
            Map<String, String> opts = Splitter.on(",").withKeyValueSeparator(":").split(maintenanceOption);
            Map<Long,Integer> optMap = new HashMap<>();
            opts.forEach((key, value) -> {
                optMap.put(Long.valueOf(key),Integer.valueOf(value));
            });
            Collection<MaintenanceOption> mtOptions = maintenanceOptionService.listByIds(optMap.keySet());
            mtOptions.forEach(option -> {
                option.setOpt(optMap.get(option.getId()));
            });
            entity.setMtOptions(mtOptions);
        }
        //查询费用项
        entity.setMtRecordCosts(mtRecordCostService.listByRecord(id));
        //查询图片
        entity.setMtRecordImgs(mtRecordImgService.listByRecord(id));
        return RestResponse.success(entity);
    }

    /**
     * 补录工单，
     * @param request
     * @return
     */
    @PostMapping("workOrder")
    public RestResponse workOrder(@RequestBody MtRecordRequest request){
        boolean result = maintenanceRecordService.workOrder(request);
        return RestResponse.success(result);
    }

    /**
     * 维保项-树形
     * @return
     */
    @PostMapping("treeOptions")
    public RestResponse treeOptions(@RequestBody MtRecordRequest request){
        Integer type = request.getType();
        Integer liftType = request.getLiftType();
        Long mtCompanyId = request.getMtCompanyId();
        Judge.ids(type,liftType,mtCompanyId);
        List<MaintenanceOption> optionList = maintenanceOptionService.listByTypeAndLiftType(type,liftType,mtCompanyId);
        Judge.notNull(optionList);
        List<Integer> sorts = optionList.stream().map(MaintenanceOption::getSort).distinct().collect(Collectors.toList());
        List<MtOptionTree> mtOptionTrees = new ArrayList<>();
        sorts.forEach(sort ->{
            List<MaintenanceOption> opts = optionList.stream().filter(option -> (sort.equals(option.getSort()))).collect(Collectors.toList());
            MtOptionTree tree = new MtOptionTree(sort,opts);
            mtOptionTrees.add(tree);
        });
        return RestResponse.success(mtOptionTrees);
    }

    /**
     * 维保项list
     * @param request
     * @return
     */
    @PostMapping("listOptions")
    public RestResponse listOptions(@RequestBody MtRecordRequest request){
        Integer type = request.getType();
        Integer liftType = request.getLiftType();
        Long mtCompanyId = request.getMtCompanyId();
        Judge.ids(type,liftType,mtCompanyId);
        List<MaintenanceOption> optionList = maintenanceOptionService.listByTypeAndLiftType(type,liftType,mtCompanyId);
        return RestResponse.success(optionList);
    }
    /**
     * 填写保养单
     * @param entity
     * @return
     */
    @PostMapping("fill")
    public RestResponse fill(@RequestBody MaintenanceRecord entity){

        return RestResponse.success();
    }
}
